/*
===============================================================================
 Name        : MSC1.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#ifdef __USE_CMSIS
#include "LPC17xx.h"
#endif

#include <cr_section_macros.h>
#include "stdlib.h"
#include "string.h"
#include "usbhost_inc.h"
#include "ff.h"
#include "ffconf.h"
#include "uart.h"

#define DEBUG_MSG   DebugWriteln

// other definitions and declarations here
FATFS Fatfs[_VOLUMES];		/* File system object for each logical drive */
uint8_t Drives;				/* Drives mask */

uint8_t Mem_Init (void);
int USBHost_Init(void);
void test_read_write();

int main(void) {
    // Force the counter to be placed into memory
    volatile static int i = 0 ;

    SystemInit();
    SystemCoreClockUpdate();

    LPC_GPIO1->FIODIR |= ((0x01 << 18) | (0x01 << 20) | (0x01 << 21) | (0x01 << 23));		// on board led
    LPC_GPIO1->FIOSET |= (0x01 << 18);
    UARTInit(1, 9600);
	DEBUG_MSG("MSC1 start.\n\r");

    Drives = Mem_Init();
    if (Drives == 0)
    {
        DEBUG_MSG ("Memory: Init failed.\n\r");
        //LCD_PrintString(0*8, 7*15, "Mem: Init Fail", COLOR_RED, COLOR_BLACK);
        goto loop;
    }

    test_read_write();

    loop:
    LPC_GPIO1->FIOSET |= (0x01 << 20);
    // Enter an infinite loop, just incrementing a counter
    while(1) {
        i++ ;
    }
    return 0 ;
}

/**
  * @brief  Initializes memory.
  *
  * @param  None.
  * @retval Mask of mounted drives
  *
  */
uint8_t Mem_Init (void)
{
    uint8_t retv = 0x0;

	/* Wait until there is USB flash disk inserted. */
    //LCD_PrintString(0*8, 4*15, "Connect USB MSD", COLOR_WHITE, COLOR_BLACK);
	if (USBHost_Init() == OK)
	{
		DEBUG_MSG("USBHost: Init OK.\n\r");
		//LCD_PrintString(0*8, 5*15, "USB: Init OK", COLOR_LIME, COLOR_BLACK);
		if (f_mount(&Fatfs[0], _T("0:"), 0) == FR_OK){
			DEBUG_MSG("USBFS: Mount OK.\n\r");
			//LCD_PrintString(0*8, 6*15, "USBFS: Mount OK", COLOR_LIME, COLOR_BLACK);
			retv |= 1;
		} else {
			DEBUG_MSG("USBFS: Mount failed.\n\r");
			//LCD_PrintString(0*8, 6*15, "USBFS: mnt fail", COLOR_RED, COLOR_BLACK);
	    }
	}
	else
	{
		DEBUG_MSG("USBHost: Init failed.\n\r");
		//LCD_PrintString(0*8, 5*15, "USB: Init fail", COLOR_RED, COLOR_BLACK);
		return retv;
	}

#ifdef SUPPORT_SD
	if (f_mount(1, &Fatfs[1]) == FR_OK) {
		DEBUG_MSG("SDFS: Mount OK.\n\r");
		LCD_PrintString(0*8, 6*15, "SDFS: Mount OK", COLOR_LIME, COLOR_BLACK);
		retv |= 2;
	} else {
		DEBUG_MSG("SDFS: Mount failed.\n\r");
		LCD_PrintString(0*8, 6*15, "SDFS: mnt fail", COLOR_RED, COLOR_BLACK);
    }
#endif

    return (retv);
}

/*----------------------------------------------------------------------------
  USB functions
 *----------------------------------------------------------------------------*/
/**
  * @brief  Initializes the USBHost .
  *
  * @param  None
  * @retval 0:OK, others: failure code
  *
  */
int USBHost_Init(void)
{
    int32_t  rc;
	uint32_t  numBlks, blkSize;
	uint8_t  inquiryResult[INQUIRY_LENGTH];

    Host_Init();               /* Initialize the lpc17xx host controller */
    rc = Host_EnumDev();       /* Enumerate the device connected  */
	if (rc != OK)
		return 1;
	rc = MS_Init( &blkSize, &numBlks, inquiryResult );
	if ((rc != OK) || (blkSize != 512))
		return 2;

	return OK;
}


void test_read_write(){
	FIL 		Fil1;
	FIL 		Fil2;
    FRESULT     res;
    UINT		br;
    UINT		br2;
    char		buff[128];

    res = f_open(&Fil1, "/MSC1_in/test.dat", FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        DEBUG_MSG ("f_open failed.\n\r");
        return ;
    }
    res = f_open(&Fil2, "/MSC1_out/testout.dat", FA_WRITE | FA_CREATE_ALWAYS);
    if (res != FR_OK){
        DEBUG_MSG ("f_open failed.\n\r");
        return ;
    }
    do {
    	res = f_read(&Fil1, buff, 128, &br);
    	if (res != FR_OK || br == 0){
    		break;
    	}
    	//DEBUG_MSG(buff);
    	res = f_write(&Fil2, buff, br, &br2);
		if (res != FR_OK){
			break;
		}
    } while (1);

    f_close(&Fil1);
    f_close(&Fil2);
    DEBUG_MSG ("test_read_write OK.\n\r");
}
